<?php

/**
 * @file
 * Contains \Drupal\uvf_migrate\Plugin\migrate\source\Term.
 */

namespace Drupal\uvf_migrate\Plugin\migrate\source;

use Drupal\migrate\Row;
use Drupal\migrate\Plugin\migrate\source\SqlBase;

/**
 * Drupal 9 taxonomy terms source from database.
 *
 * @todo Support term_relation, term_synonym table if possible.
 *
 * @MigrateSource(
 *   id = "custom_taxonomy_term",
 *   source_provider = "taxonomy"
 * )
 */
class Term extends SqlBase
{
    /**
     * {@inheritdoc}
     */
    public function query()
    {
        $query = $this->select('taxonomy_term_field_data', 'td')
            ->fields('td', [
                'tid',
                'vid',
                'name',
                'description__value',
                'weight'
            ]);
            $query->addField('tp', 'parent_target_id');
            $query->leftjoin('taxonomy_term__parent', 'tp', 'td.tid = tp.entity_id');
            $query->distinct();
            
        return $query;
    }

    /**
     * {@inheritdoc}
     */
    public function fields()
    {
        return [
            'tid' => $this->t('The term ID.'),
            'vid' => $this->t('Existing term VID'),
            'name' => $this->t('The name of the term.'),
            'description__value' => $this->t('The term description.'),
            'weight' => $this->t('Weight'),
            'parent_target_id' => $this->t("The Drupal term IDs of the term's parents."),
        ];
    }

   

    /**
     * {@inheritdoc}
     */
    public function getIds()
    {
        $ids['tid']['type'] = 'integer';
        return $ids;
    }
}
